//Josip Medved <jmedved@jmedved.com> http://www.jmedved.com


using System;
using System.Runtime.InteropServices;

namespace Medo.Configuration {

    /// <summary>
    /// Controlling power requirements.
    /// </summary>
    public class PowerRequirement : IDisposable {

        private IntPtr _handle;
        private string _deviceName;

        /// <summary>
        /// Creates new instance.
        /// </summary>
        public PowerRequirement(string deviceName) {
            this._deviceName = deviceName;
        }

        ~PowerRequirement() {
            Dispose(false);
        }


        /// <summary>
        /// First audio device.
        /// </summary>
        public const string AudioDevice = "WAV1:";

        /// <summary>
        /// First backlight device.
        /// </summary>
        public const string BacklightDevice = "BKL1:";

        /// <summary>
        /// First serial port device.
        /// </summary>
        public const string SerialPortDevice = "COM1:";


        /// <summary>
        /// Sets power requirement.
        /// </summary>
        /// <exception cref="System.InvalidOperationException">Device already open. -or- Cannot open device.</exception>
        public void Set(PowerRequirementState powerState) {
            if (this._handle != IntPtr.Zero) { throw new System.InvalidOperationException("Device already open."); }

            this._handle = NativeMethods.SetPowerRequirement(this._deviceName, powerState, NativeMethods.POWER_NAME, IntPtr.Zero, 0);
            if (this._handle == IntPtr.Zero) { throw new System.InvalidOperationException("Cannot open device."); }
        }

        /// <summary>
        /// Gets whether power requirement is set.
        /// </summary>
        public bool IsSet {
            get { return (this._handle != IntPtr.Zero); }
        }

        /// <summary>
        /// Releases power requirement.
        /// </summary>
        /// <exception cref="System.InvalidOperationException">Cannot close device.</exception>
        public void Release() {
            if (this._handle != IntPtr.Zero) {
                if (NativeMethods.ReleasePowerRequirement(this._handle) == NativeMethods.ERROR_SUCCESS) {
                    this._handle = IntPtr.Zero;
                } else {
                    throw new System.InvalidOperationException("Cannot close device.");
                }
            }
        }


        #region IDisposable Members

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">True if managed resources should be disposed; otherwise, false.</param>
        protected virtual void Dispose(bool disposing) {
            if (disposing) { }
            this.Release();
        }


        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        public void Dispose() {
            Dispose(true);
            System.GC.SuppressFinalize(this);
        }

        #endregion


        private static class NativeMethods {

            internal const Int32 ERROR_SUCCESS = 0;
            internal const Int32 POWER_NAME = 1;
            internal const Int32 POWER_FORCE = 2;


            [DllImport("coredll.dll")]
            internal static extern IntPtr SetPowerRequirement(String pvDevice, PowerRequirementState deviceState, Int32 DeviceFlags, IntPtr pvSystemState, Int32 StateFlags);

            [DllImport("coredll.dll")]
            internal static extern Int32 ReleasePowerRequirement(IntPtr hPowerReq);

        }

    }



    /// <summary>
    /// Power states.
    /// </summary>
    public enum PowerRequirementState {

        /// <summary>
        /// The system state is unspecified. 
        /// </summary>
        Unspecified = -1,
        /// <summary>
        /// Full On. This is the state in which the device is on and running. It is receiving full power from the system and is delivering full functionality to the user.
        /// </summary>
        D0 = 0,
        /// <summary>
        /// Low On. This is the state in which the device is fully functional at a lower power state than D0, a lower performance state than D0, or both. This state is applicable when the device is being used, but where peak performance is unnecessary and power is at a premium.
        /// </summary>
        D1 = 1,
        /// <summary>
        /// Standby. This is the state in which the device is partially powered with automatic wakeup on request. The device is effectively standing by.
        /// </summary>
        D2 = 2,
        /// <summary>
        /// Sleep. This is the state in which the device is partially powered with device-initiated wakeup if available. A device in state D3 is sleeping but capable of raising the System Power State on its own. It consumes only enough power to be able to do so; which must be less than or equal to the amount of power used in state D2.
        /// </summary>
        D3 = 3,
        /// <summary>
        /// Off. This is the state in which the device is not powered. A device in state D4 should not be consuming any significant power. Some peripheral busses require static terminations that intrinsically use non-zero power when a device is physically connected to the bus; a device on such a bus can still support D4.
        /// </summary>
        D4 = 4,
    }

}
